@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2007 West Consulting
-   Copyright (C) 2007 Arthur de Jong
+   Copyright (C) 2007, 2008 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -43,6 +43,9 @@
 #include <gssapi/gssapi.h>
 #include <gssapi/gssapi_krb5.h>
 #endif /* HAVE_GSSAPI_GSSAPI_KRB5_H */
+#include <sys/types.h>
+#include <pwd.h>
+#include <grp.h>
 
 #include "common.h"
 #include "log.h"
@@ -62,8 +65,15 @@ static void cfg_defaults(struct ldap_config *cfg)
 {
   int i;
   memset(cfg,0,sizeof(struct ldap_config));
+  cfg->ldc_threads=5;
+  cfg->ldc_uid=NOUID;
+  cfg->ldc_gid=NOGID;
   for (i=0;i<(NSS_LDAP_CONFIG_URI_MAX+1);i++)
-    cfg->ldc_uris[i]=NULL;
+  {
+    cfg->ldc_uris[i].uri=NULL;
+    cfg->ldc_uris[i].lastok=0;
+    cfg->ldc_uris[i].lastfail=0;
+  }
 #ifdef LDAP_VERSION3
   cfg->ldc_version=LDAP_VERSION3;
 #else /* LDAP_VERSION3 */
@@ -71,13 +81,12 @@ static void cfg_defaults(struct ldap_config *cfg)
 #endif /* not LDAP_VERSION3 */
   cfg->ldc_binddn=NULL;
   cfg->ldc_bindpw=NULL;
-  cfg->ldc_rootbinddn=NULL;
-  cfg->ldc_rootbindpw=NULL;
-  cfg->ldc_saslid=NULL;
-  cfg->ldc_rootsaslid=NULL;
+  cfg->ldc_sasl_authcid=NULL;
+  cfg->ldc_sasl_authzid=NULL;
   cfg->ldc_sasl_secprops=NULL;
+  cfg->ldc_sasl_mech=NULL;
+  cfg->ldc_sasl_realm=NULL;
   cfg->ldc_usesasl=0;
-  cfg->ldc_rootusesasl=0;
   cfg->ldc_base=NULL;
   cfg->ldc_scope=LDAP_SCOPE_SUBTREE;
   cfg->ldc_deref=LDAP_DEREF_NEVER;
@@ -85,9 +94,9 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->ldc_bind_timelimit=30;
   cfg->ldc_timelimit=LDAP_NO_LIMIT;
   cfg->ldc_idle_timelimit=0;
-  cfg->ldc_reconnect_tries=4;
   cfg->ldc_reconnect_sleeptime=1;
   cfg->ldc_reconnect_maxsleeptime=30;
+#ifdef LDAP_OPT_X_TLS
   cfg->ldc_ssl_on=SSL_OFF;
   cfg->ldc_sslpath=NULL;
   cfg->ldc_tls_checkpeer=-1;
@@ -97,6 +106,7 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->ldc_tls_ciphers=NULL;
   cfg->ldc_tls_cert=NULL;
   cfg->ldc_tls_key=NULL;
+#endif /* LDAP_OPT_X_TLS */
   cfg->ldc_restart=1;
   cfg->ldc_pagesize=0;
   cfg->ldc_debug=0;
@@ -127,7 +137,7 @@ static void add_uri(const char *filename,int lnr,
   int i;
   log_log(LOG_DEBUG,"add_uri(%s)",uri);
   /* find the place where to insert the URI */
-  for (i=0;cfg->ldc_uris[i]!=NULL;i++)
+  for (i=0;cfg->ldc_uris[i].uri!=NULL;i++)
     ;
   /* check for room */
   if (i>=NSS_LDAP_CONFIG_URI_MAX)
@@ -136,10 +146,14 @@ static void add_uri(const char *filename,int lnr,
     exit(EXIT_FAILURE);
   }
   /* append URI to list */
-  cfg->ldc_uris[i]=xstrdup(uri);
-  cfg->ldc_uris[i+1]=NULL;
+  cfg->ldc_uris[i].uri=xstrdup(uri);
 }
 
+#ifndef HOST_NAME_MAX
+#define HOST_NAME_MAX 255
+#endif /* not HOST_NAME_MAX */
+
+#ifdef HAVE_LDAP_DOMAIN2HOSTLIST
 /* return the domain name of the current host
    we return part of the structure that is retured by gethostbyname()
    so there should be no need to free() this entry, however we should
@@ -171,7 +185,7 @@ static const char *cfg_getdomainname(const char *filename,int lnr)
 
 /* add URIs by doing DNS queries for SRV records */
 static void add_uris_from_dns(const char *filename,int lnr,
-                        struct ldap_config *cfg)
+                              struct ldap_config *cfg)
 {
   int ret=0;
   const char *domain;
@@ -204,6 +218,7 @@ static void add_uris_from_dns(const char *filename,int lnr,
     hostlist=nxt;
   }
 }
+#endif /* HAVE_LDAP_DOMAIN2HOSTLIST */
 
 static int parse_boolean(const char *filename,int lnr,const char *value)
 {
@@ -329,6 +344,116 @@ static inline void check_argumentcount(const char *filename,int lnr,
   }
 }
 
+static void get_int(const char *filename,int lnr,
+                    const char *keyword,char **line,
+                    int *var)
+{
+  /* TODO: refactor to have less overhead */
+  char token[32];
+  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
+  /* TODO: replace with correct numeric parse */
+  *var=atoi(token);
+}
+
+static void get_boolean(const char *filename,int lnr,
+                        const char *keyword,char **line,
+                        int *var)
+{
+  /* TODO: refactor to have less overhead */
+  char token[32];
+  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
+  *var=parse_boolean(filename,lnr,token);
+}
+
+static void get_strdup(const char *filename,int lnr,
+                       const char *keyword,char **line,
+                       char **var)
+{
+  /* TODO: refactor to have less overhead */
+  char token[64];
+  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
+  if ((*var==NULL)||(strcmp(*var,token)!=0))
+  {
+    /* Note: we have a memory leak here if a single mapping is changed
+             multiple times in one config (deemed not a problem) */
+    *var=xstrdup(token);
+  }
+}
+
+static void get_restdup(const char *filename,int lnr,
+                        const char *keyword,char **line,
+                        char **var)
+{
+  check_argumentcount(filename,lnr,keyword,(*line!=NULL)&&(**line!='\0'));
+  if ((*var==NULL)||(strcmp(*var,*line)!=0))
+  {
+    /* Note: we have a memory leak here if a single mapping is changed
+             multiple times in one config (deemed not a problem) */
+    *var=xstrdup(*line);
+  }
+  *line=NULL;
+}
+
+static void get_eol(const char *filename,int lnr,
+                    const char *keyword,char **line)
+{
+  if ((line!=NULL)&&(*line!=NULL)&&(**line!='\0'))
+  {
+    log_log(LOG_ERR,"%s:%d: %s: too may arguments",filename,lnr,keyword);
+    exit(EXIT_FAILURE);
+  }
+}
+
+static void get_uid(const char *filename,int lnr,
+                    const char *keyword,char **line,
+                    uid_t *var)
+{
+  /* TODO: refactor to have less overhead */
+  char token[32];
+  struct passwd *pwent;
+  char *tmp;
+  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
+  /* check if it is a valid numerical uid */
+  *var=(uid_t)strtol(token,&tmp,0);
+  if ((*token!='\0')&&(*tmp=='\0'))
+    return;
+  /* find by name */
+  pwent=getpwnam(token);
+  if (pwent!=NULL)
+  {
+    *var=pwent->pw_uid;
+    return;
+  }
+  /* log an error */
+  log_log(LOG_ERR,"%s:%d: %s: not a valid uid: '%s'",filename,lnr,keyword,token);
+  exit(EXIT_FAILURE);
+}
+
+static void get_gid(const char *filename,int lnr,
+                    const char *keyword,char **line,
+                    gid_t *var)
+{
+  /* TODO: refactor to have less overhead */
+  char token[32];
+  struct group *grent;
+  char *tmp;
+  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
+  /* check if it is a valid numerical gid */
+  *var=(gid_t)strtol(token,&tmp,0);
+  if ((*token!='\0')&&(*tmp=='\0'))
+    return;
+  /* find by name */
+  grent=getgrnam(token);
+  if (grent!=NULL)
+  {
+    *var=grent->gr_gid;
+    return;
+  }
+  /* log an error */
+  log_log(LOG_ERR,"%s:%d: %s: not a valid gid: '%s'",filename,lnr,keyword,token);
+  exit(EXIT_FAILURE);
+}
+
 static void parse_krb5_ccname_statement(const char *filename,int lnr,
                                         const char *keyword,char *line)
 {
@@ -337,7 +462,9 @@ static void parse_krb5_ccname_statement(const char *filename,int lnr,
   const char *ccfile;
   size_t ccenvlen;
   char *ccenv;
+#ifdef HAVE_GSS_KRB5_CCACHE_NAME
   OM_uint32 minor_status;
+# endif /* HAVE_GSS_KRB5_CCACHE_NAME */
   /* get token */
   check_argumentcount(filename,lnr,keyword,
       (get_token(&line,token,sizeof(token))!=NULL)&&(*line=='\0'));
@@ -380,13 +507,20 @@ static void parse_krb5_ccname_statement(const char *filename,int lnr,
 static void set_base(const char *filename,int lnr,
                      const char *value,const char **var)
 {
+#ifdef HAVE_LDAP_DOMAIN2DN
   char *domaindn=NULL;
+#endif /* HAVE_LDAP_DOMAIN2DN */
   /* if the base is "DOMAIN" use the domain name */
   if (strcasecmp(value,"domain")==0)
   {
+#ifdef HAVE_LDAP_DOMAIN2DN
     ldap_domain2dn(cfg_getdomainname(filename,lnr),&domaindn);
     log_log(LOG_DEBUG,"set_base(): setting base to %s from domain",domaindn);
     value=domaindn;
+#else /* not HAVE_LDAP_DOMAIN2DN */
+    log_log(LOG_ERR,"%s:%d: value %s not supported on platform",filename,lnr,value);
+    exit(EXIT_FAILURE);
+#endif /* not HAVE_LDAP_DOMAIN2DN */
   }
   /* check if the value will be changed */
   if ((*var==NULL)||(strcmp(*var,value)!=0))
@@ -425,10 +559,11 @@ static void parse_filter_statement(const char *filename,int lnr,
                                    const char *keyword,char *line)
 {
   const char **var;
+  const char *map=line;
   var=filter_get_var(get_map(&line));
   if (var==NULL)
   {
-    log_log(LOG_ERR,"%s:%d: unknown map: '%s'",filename,lnr,line);
+    log_log(LOG_ERR,"%s:%d: unknown map: '%s'",filename,lnr,map);
     exit(EXIT_FAILURE);
   }
   check_argumentcount(filename,lnr,keyword,(line!=NULL)&&(*line!='\0'));
@@ -458,7 +593,8 @@ static void parse_map_statement(const char *filename,int lnr,
   check_argumentcount(filename,lnr,keyword,
       (get_token(&line,oldatt,sizeof(oldatt))!=NULL)&&
       (get_token(&line,newatt,sizeof(newatt))!=NULL));
-  check_argumentcount(filename,lnr,keyword,(line!=NULL)&&(*line!='\0'));
+  /* check that there are no more tokens left on the line */
+  get_eol(filename,lnr,keyword,&line);
   /* get the attribute variable to set */
   var=attmap_get_var(map,oldatt);
   if (var==NULL)
@@ -467,7 +603,7 @@ static void parse_map_statement(const char *filename,int lnr,
     exit(EXIT_FAILURE);
   }
   /* check if the value will be changed */
-  if (strcmp(*var,newatt)!=0)
+  if ( (*var==NULL) || (strcmp(*var,newatt)!=0) )
   {
     /* Note: we have a memory leak here if a single mapping is changed
              multiple times in one config (deemed not a problem) */
@@ -475,66 +611,6 @@ static void parse_map_statement(const char *filename,int lnr,
   }
 }
 
-static void get_int(const char *filename,int lnr,
-                    const char *keyword,char **line,
-                    int *var)
-{
-  /* TODO: refactor to have less overhead */
-  char token[32];
-  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
-  /* TODO: replace with correct numeric parse */
-  *var=atoi(token);
-}
-
-static void get_boolean(const char *filename,int lnr,
-                        const char *keyword,char **line,
-                        int *var)
-{
-  /* TODO: refactor to have less overhead */
-  char token[32];
-  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
-  *var=parse_boolean(filename,lnr,token);
-}
-
-static void get_strdup(const char *filename,int lnr,
-                    const char *keyword,char **line,
-                    char **var)
-{
-  /* TODO: refactor to have less overhead */
-  char token[64];
-  check_argumentcount(filename,lnr,keyword,get_token(line,token,sizeof(token))!=NULL);
-  if ((*var==NULL)||(strcmp(*var,token)!=0))
-  {
-    /* Note: we have a memory leak here if a single mapping is changed
-             multiple times in one config (deemed not a problem) */
-    *var=xstrdup(token);
-  }
-}
-
-static void get_restdup(const char *filename,int lnr,
-                     const char *keyword,char **line,
-                     char **var)
-{
-  check_argumentcount(filename,lnr,keyword,(*line!=NULL)&&(**line!='\0'));
-  if ((*var==NULL)||(strcmp(*var,*line)!=0))
-  {
-    /* Note: we have a memory leak here if a single mapping is changed
-             multiple times in one config (deemed not a problem) */
-    *var=xstrdup(*line);
-  }
-  *line=NULL;
-}
-
-static void get_eol(const char *filename,int lnr,
-                   const char *keyword,char **line)
-{
-  if ((line!=NULL)&&(*line!=NULL)&&(**line!='\0'))
-  {
-    log_log(LOG_ERR,"%s:%d: %s: too may arguments",filename,lnr,keyword);
-    exit(EXIT_FAILURE);
-  }
-}
-
 static void cfg_read(const char *filename,struct ldap_config *cfg)
 {
   FILE *fp;
@@ -572,14 +648,37 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
     /* get keyword from line and ignore empty lines */
     if (get_token(&line,keyword,sizeof(keyword))==NULL)
       continue;
+    /* runtime options */
+    if (strcasecmp(keyword,"threads")==0)
+    {
+      get_int(filename,lnr,keyword,&line,&cfg->ldc_threads);
+      get_eol(filename,lnr,keyword,&line);
+    }
+    else if (strcasecmp(keyword,"uid")==0)
+    {
+      get_uid(filename,lnr,keyword,&line,&cfg->ldc_uid);
+      get_eol(filename,lnr,keyword,&line);
+    }
+    else if (strcasecmp(keyword,"gid")==0)
+    {
+      get_gid(filename,lnr,keyword,&line,&cfg->ldc_gid);
+      get_eol(filename,lnr,keyword,&line);
+    }
     /* general connection options */
-    if (strcasecmp(keyword,"uri")==0)
+    else if (strcasecmp(keyword,"uri")==0)
     {
       check_argumentcount(filename,lnr,keyword,(line!=NULL)&&(*line!='\0'));
       while (get_token(&line,token,sizeof(token))!=NULL)
       {
         if (strcasecmp(token,"dns")==0)
+        {
+#ifdef HAVE_LDAP_DOMAIN2HOSTLIST
           add_uris_from_dns(filename,lnr,cfg);
+#else /* not HAVE_LDAP_DOMAIN2HOSTLIST */
+          log_log(LOG_ERR,"%s:%d: value %s not supported on platform",filename,lnr,token);
+          exit(EXIT_FAILURE);
+#endif /* not HAVE_LDAP_DOMAIN2HOSTLIST */
+        }
         else
           add_uri(filename,lnr,cfg,token);
       }
@@ -597,51 +696,43 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
     {
       get_restdup(filename,lnr,keyword,&line,&cfg->ldc_bindpw);
     }
-    else if (strcasecmp(keyword,"rootbinddn")==0)
-    {
-      log_log(LOG_ERR,"%s:%d: option %s is currently unsupported",filename,lnr,keyword);
-      exit(EXIT_FAILURE);
-      get_restdup(filename,lnr,keyword,&line,&cfg->ldc_rootbinddn);
-    }
-    else if (strcasecmp(keyword,"rootbindpw")==0)
-    {
-      log_log(LOG_ERR,"%s:%d: option %s is currently unsupported",filename,lnr,keyword);
-      exit(EXIT_FAILURE);
-      get_restdup(filename,lnr,keyword,&line,&cfg->ldc_rootbinddn);
-    }
     /* SASL authentication options */
-    else if (strcasecmp(keyword,"sasl_authid")==0)
+    else if (strcasecmp(keyword,"sasl_authcid")==0)
     {
-      log_log(LOG_WARNING,"%s:%d: option %s is currently untested (please report any successes)",filename,lnr,keyword);
-      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_saslid);
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
+      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_sasl_authcid);
       get_eol(filename,lnr,keyword,&line);
     }
-    else if (strcasecmp(keyword,"rootsasl_authid")==0)
+    else if (strcasecmp(keyword,"sasl_authzid")==0)
     {
-      log_log(LOG_ERR,"%s:%d: option %s is currently unsupported",filename,lnr,keyword);
-      exit(EXIT_FAILURE);
-      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_rootsaslid);
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
+      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_sasl_authzid);
+      get_eol(filename,lnr,keyword,&line);
+    }
+    else if (strcasecmp(keyword,"sasl_mech")==0)
+    {
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
+      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_sasl_mech);
+      get_eol(filename,lnr,keyword,&line);
+    }
+    else if (strcasecmp(keyword,"sasl_realm")==0)
+    {
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
+      get_strdup(filename,lnr,keyword,&line,&cfg->ldc_sasl_realm);
       get_eol(filename,lnr,keyword,&line);
     }
     else if (strcasecmp(keyword,"sasl_secprops")==0)
     {
-      log_log(LOG_WARNING,"%s:%d: option %s is currently untested (please report any successes)",filename,lnr,keyword);
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
       get_strdup(filename,lnr,keyword,&line,&cfg->ldc_sasl_secprops);
       get_eol(filename,lnr,keyword,&line);
     }
     else if (strcasecmp(keyword,"use_sasl")==0)
     {
-      log_log(LOG_WARNING,"%s:%d: option %s is currently untested (please report any successes)",filename,lnr,keyword);
+      log_log(LOG_WARNING,"%s:%d: option %s is currently not fully supported (please report any successes)",filename,lnr,keyword);
       get_boolean(filename,lnr,keyword,&line,&cfg->ldc_usesasl);
       get_eol(filename,lnr,keyword,&line);
     }
-    else if (strcasecmp(keyword,"rootuse_sasl")==0)
-    {
-      log_log(LOG_ERR,"%s:%d: option %s is currently unsupported",filename,lnr,keyword);
-      exit(EXIT_FAILURE);
-      get_boolean(filename,lnr,keyword,&line,&cfg->ldc_rootusesasl);
-      get_eol(filename,lnr,keyword,&line);
-    }
     /* Kerberos authentication options */
     else if (strcasecmp(keyword,"krb5_ccname")==0)
     {
@@ -705,10 +796,7 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
       get_eol(filename,lnr,keyword,&line);
     }
     else if (strcasecmp(keyword,"reconnect_tries")==0)
-    {
-      get_int(filename,lnr,keyword,&line,&cfg->ldc_reconnect_tries);
-      get_eol(filename,lnr,keyword,&line);
-    }
+      log_log(LOG_WARNING,"%s:%d: option %s has been removed and will be ignored",filename,lnr,keyword);
     else if (!strcasecmp(keyword,"reconnect_sleeptime"))
     {
       get_int(filename,lnr,keyword,&line,&cfg->ldc_reconnect_sleeptime);
@@ -719,10 +807,10 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
       get_int(filename,lnr,keyword,&line,&cfg->ldc_reconnect_maxsleeptime);
       get_eol(filename,lnr,keyword,&line);
     }
+#ifdef LDAP_OPT_X_TLS
     /* SSL/TLS options */
     else if (strcasecmp(keyword,"ssl")==0)
     {
-      log_log(LOG_WARNING,"%s:%d: option %s is currently untested (please report any successes)",filename,lnr,keyword);
       check_argumentcount(filename,lnr,keyword,
           (get_token(&line,token,sizeof(token))!=NULL));
       if (strcasecmp(token,"start_tls")==0)
@@ -784,6 +872,7 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
       get_eol(filename,lnr,keyword,&line);
       /* TODO: check that the path is valid */
     }
+#endif /* LDAP_OPT_X_TLS */
     /* other options */
     else if (strcasecmp(keyword,"restart")==0)
     {
@@ -807,8 +896,60 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
   fclose(fp);
 }
 
+/* This function tries to get the LDAP search base from the LDAP server.
+   Note that this returns a string that has been allocated with strdup().
+   For this to work the myldap module needs enough configuration information
+   to make an LDAP connection. */
+static MUST_USE char *get_base_from_rootdse(void)
+{
+  MYLDAP_SESSION *session;
+  MYLDAP_SEARCH *search;
+  MYLDAP_ENTRY *entry;
+  const char *attrs[] = { "+", NULL };
+  int i;
+  int rc;
+  const char **values;
+  char *base=NULL;
+  /* initialize session */
+  session=myldap_create_session();
+  assert(session!=NULL);
+  /* perform search */
+  search=myldap_search(session,"",LDAP_SCOPE_BASE,"(objectClass=*)",attrs);
+  if (search==NULL)
+  {
+    myldap_session_close(session);
+    return NULL;
+  }
+  /* go over results */
+  for (i=0;(entry=myldap_get_entry(search,&rc))!=NULL;i++)
+  {
+    /* get defaultNamingContext */
+    values=myldap_get_values(entry,"defaultNamingContext");
+    if ((values!=NULL)&&(values[0]!=NULL))
+    {
+      base=xstrdup(values[0]);
+      log_log(LOG_DEBUG,"get_basedn_from_rootdse(): found attribute defaultNamingContext with value %s",values[0]);
+      break;
+    }
+    /* get namingContexts */
+    values=myldap_get_values(entry,"namingContexts");
+    if ((values!=NULL)&&(values[0]!=NULL))
+    {
+      base=xstrdup(values[0]);
+      log_log(LOG_DEBUG,"get_basedn_from_rootdse(): found attribute namingContexts with value %s",values[0]);
+      break;
+    }
+  }
+  /* clean up */
+  myldap_session_close(session);
+  return base;
+}
+
 void cfg_init(const char *fname)
 {
+#ifdef LDAP_OPT_X_TLS
+  int i;
+#endif /* LDAP_OPT_X_TLS */
   /* check if we were called before */
   if (nslcd_cfg!=NULL)
   {
@@ -827,9 +968,32 @@ void cfg_init(const char *fname)
   /* read configfile */
   cfg_read(fname,nslcd_cfg);
   /* do some sanity checks */
-  if (nslcd_cfg->ldc_uris[0]==NULL)
+  if (nslcd_cfg->ldc_uris[0].uri==NULL)
   {
     log_log(LOG_ERR,"no URIs defined in config");
     exit(EXIT_FAILURE);
   }
+  /* if ssl is on each URI should start with ldaps */
+#ifdef LDAP_OPT_X_TLS
+  if (nslcd_cfg->ldc_ssl_on==SSL_LDAPS)
+  {
+    for (i=0;nslcd_cfg->ldc_uris[i].uri!=NULL;i++)
+    {
+      if (strncasecmp(nslcd_cfg->ldc_uris[i].uri,"ldaps://",8)!=0)
+        log_log(LOG_WARNING,"%s doesn't start with ldaps:// and \"ssl on\" is specified",
+                            nslcd_cfg->ldc_uris[i].uri);
+    }
+  }
+  /* TODO: check that if some tls options are set the ssl option should be set to on (just warn) */
+#endif /* LDAP_OPT_X_TLS */
+  /* if basedn is not yet set,  get if from the rootDSE */
+  if (nslcd_cfg->ldc_base==NULL)
+    nslcd_cfg->ldc_base=get_base_from_rootdse();
+  /* TODO: handle the case gracefully when no LDAP server is available yet */
+  /* see if we have a valid basedn */
+  if ((nslcd_cfg->ldc_base==NULL)||(nslcd_cfg->ldc_base[0]=='\0'))
+  {
+    log_log(LOG_ERR,"no base defined in config and couldn't get one from server");
+    exit(EXIT_FAILURE);
+  }
 }
